# HPC for Data Analytics

With the HPC-DA system, the TU Dresden provides infrastructure for
High-Performance Computing and Data Analytics (HPC-DA) for German
researchers for computing projects with focus in one of the following
areas:

-   machine learning scenarios for large systems
-   evaluation of various hardware settings for large machine learning
    problems, including accelerator and compute node configuration and
    memory technologies
-   processing of large amounts of data on highly parallel machine
    learning infrastructure.

Currently we offer 25 Mio core hours compute time per year for external
computing projects. Computing projects have a duration of up to one year
with the possibility of extensions, thus enabling projects to continue
seamlessly. Applications for regular projects on HPC-DA can be submitted
at any time via the \<a
href="<https://tu-dresden.de/zih/hochleistungsrechnen/zugang/hpc-da>"
target="new">online web-based submission\</a> and review system. The
reviews of the applications are carried out by experts in their
respective scientific fields. Applications are evaluated only according
to their scientific excellence.

ZIH provides a portfolio of preinstalled applications and offers support
for software installation/configuration of project-specific
applications. In particular, we provide consulting services for all our
users, and advise researchers on using the resources in an efficient
way.

\<img align="right" alt="HPC-DA Overview"
src="%ATTACHURL%/bandwidth.png" title="bandwidth.png" width="250" />

## Access

-   Application for access using this \<a class="TMLlink"
    href="<https://tu-dresden.de/zih/hochleistungsrechnen/zugang/hpc-da>"
    target="new">Online Web Form\</a>

## Hardware Overview

-   [Nodes for machine learning (Power9)](Power9)
-   [NVMe Storage](NvmeStorage) (2 PB)
-   \<a href="HpcdaWarmArchive" title="Warm Archive">Warm archive\</a>
    (10 PB)
-   HPC nodes (x86) for DA (island 6)
-   Compute nodes with high memory bandwidth: [AMD Rome
    Nodes](RomeNodes) (island 7)

Additional hardware:

-   [Multi-GPU-Cluster](AlphaCentauri) for projects of SCADS.AI

## File Systems and Object Storage

-   Lustre
-   BeeGFS
-   Quobyte
-   S3

## HOWTOS

-   [Get started with HPC-DA](GetStartedWithHPCDA)
-   [IBM Power AI](PowerAI)
-   [Work with Singularity Containers on Power9\<br />](Cloud)
-   [TensorFlow on HPC-DA (native)](TensorFlow)
-   \<a href="TensorFlowOnJupyterNotebook" target="\_blank">Tensorflow
    on Jupyter notebook\</a>
-   Create and run your own TensorFlow container for HPC-DA (Power9)
-   [TensorFlow on x86](DeepLearning)
-   \<a href="PyTorch" target="\_blank">PyTorch on HPC-DA (Power9)\</a>
-   \<a href="Python" target="\_blank">Python on HPC-DA (Power9)\</a>
-   [JupyterHub](JupyterHub)
-   [R on HPC-DA (Power9)](DataAnalyticsWithR)
-   [Big Data frameworks: Apache Spark, Apache Flink, Apache
    Hadoop](BigDataFrameworks:ApacheSparkApacheFlinkApacheHadoop)

-- Main.UlfMarkwardt - 2019-05-05
